package com.honksandsirens;
import com.softsynth.jsyn.*;

/**
 * James Forrest's Format Filter, modified with setStage() triggering vowels, 
 * and a rate port to control transition speed.
 * 
 * You can connect any signal source to the input port of this circuit.  
 * 
 * Keep the amplitude of StagedVowelizer high, around 10
 * 
 * use setStage(time, stage) to trigger a vowel.
 * 
 * Use rate port to set speed of transition between vowels, where rate = 1 transitions in 0.1 seconds. 
 * transition speed in seconds = 0.1/rate
 * 
 * @author Nick Didkovsky, didkovn@mail.rockefeller.edu
*/


public class StagedVowelizer extends SynthCircuit
{    

	public final static int NUM_FILTERS = 5;
	public final static int NUM_FORMANTS = 8;
	
	public static final int A = 0;
	public static final int E = 1;
	public static final int I = 2;
	public static final int O = 3;
	public static final int OO = 4;
	public static final int U = 5;
	public static final int ER = 6;
	public static final int UH = 7;
	
        int lastDataLength=0;

	Formant formants[] = new Formant[NUM_FORMANTS];	
	
    // Declare units and ports.
    FilterBank 			formantFilters;
    EnvelopePlayer		freqEnvelopes[];
    EnvelopePlayer		ampEnvelopes[];
    SynthEnvelope		freqSynthEnv[];
    SynthEnvelope		ampSynthEnv[];
    BusWriter  			filterOuts[];
    BusReader  			filterAdder;
    MultiplyUnit		multiplier;
    public SynthInput	input;
    /** Controls speed of transition from vowel to vowel.  Range 0..2 . Transition speed in seconds = 0.1/rate*/
    public SynthDistributor	rate;
    public SynthInput	amplitude;
    
    public StagedVowelizer() {
    	formants[A] = new Formant(Formant.A);
	formants[E] = new Formant(Formant.E);
	formants[I] = new Formant(Formant.I);
	formants[O] = new Formant(Formant.O);
	formants[OO] = new Formant(Formant.OO);
	formants[U] = new Formant(Formant.U);
	formants[ER] = new Formant(Formant.ER);
	formants[UH] = new Formant(Formant.UH);
    	
    	add( formantFilters = new FilterBank(NUM_FILTERS) );
    	rate = new SynthDistributor( "rate" );
    	freqEnvelopes = new EnvelopePlayer[NUM_FILTERS];
    	freqSynthEnv = new SynthEnvelope[NUM_FILTERS];
    	ampSynthEnv = new SynthEnvelope[NUM_FILTERS];
    	ampEnvelopes = new EnvelopePlayer[NUM_FILTERS];
    	filterOuts = new BusWriter[NUM_FILTERS];
    	
    	add( filterAdder = new BusReader() );
    	
    	for (int i=0; i<NUM_FILTERS; i++) {
    		add( filterOuts[i] = new BusWriter());
    		formantFilters.outputs[i].connect( filterOuts[i].input );
    		add( freqEnvelopes[i] = new EnvelopePlayer());
    		rate.connect(freqEnvelopes[i].rate);
    		freqEnvelopes[i].output.connect(formantFilters.freqs[i]);
    		add( ampEnvelopes[i] = new EnvelopePlayer());
    		rate.connect(ampEnvelopes[i].rate);
    		ampEnvelopes[i].output.connect(formantFilters.amps[i]);
    		formantFilters.Qs[i].set(10.0);
    		filterOuts[i].busOutput.connect( filterAdder.busInput );
    	}
    	add( multiplier = new MultiplyUnit() );
    	filterAdder.output.connect(multiplier.inputA);
    	addPort( amplitude = multiplier.inputB, "amplitude" );
    	addPort( input = formantFilters.input, "input" );
    	addPort( output = multiplier.output, "output" );
    	amplitude.setup(0, 0.5, 10.0);
    	rate.set(1.0);
    	double[] defaultData = { 0.2, A, 1.3, OO };
    	buildEnvelopes(defaultData);
    }
    
    /** pass in a data array, containing duration format_index pairs 
    
    ex. { 1.0,  A, 1.5, E }
    
    Build envelopes on  the fly which deliver this.
    */
    
    
    public void buildEnvelopes(double[] data) {
    	double[] freqEnvData = new double[data.length];
    	double[] ampEnvData = new double[data.length];
    	for (int i=0; i<NUM_FILTERS; i++) {
    		for (int j=0; j<data.length/2; j++) {
    			double dur = data[j*2];
    			int indexOfFormant = (int)data[j*2+1];
    			freqEnvData[j*2] = dur;
    			ampEnvData[j*2] = dur;
	    		freqEnvData[j*2+1] = formants[indexOfFormant].getFrequency(i);
        		ampEnvData[j*2+1] = formants[indexOfFormant].getAmplitude(i);
        	}
        	if (data.length != lastDataLength) {
        		freqSynthEnv[i] = new SynthEnvelope(freqEnvData);
        		ampSynthEnv[i] = new SynthEnvelope(ampEnvData);
        	} else {
        		freqSynthEnv[i].write(freqEnvData);
        		ampSynthEnv[i].write(ampEnvData);
        	}
        }
        lastDataLength = data.length;
    }
    
    double[] vowels = new double[4];

    /** stage 0..7 queues up vowel: A, E, I, O, OO, U, ER, UH */
    public void setStage(int time, int stage) {
	vowels[0] = 0.1;
	vowels[2] = 0.1;
	vowels[1] = stage;
	vowels[3] = stage;
        buildEnvelopes(vowels);

        for (int i=0; i<NUM_FILTERS; i++) {
            freqEnvelopes[i].envelopePort.clear(time);
            ampEnvelopes[i].envelopePort.clear(time);
            freqEnvelopes[i].envelopePort.queue( time, freqSynthEnv[i], 0, freqSynthEnv[i].getNumFrames());
            ampEnvelopes[i].envelopePort.queue( time, ampSynthEnv[i], 0, ampSynthEnv[i].getNumFrames());
        }
    }
    
    public static void main( String args[] ) {
    	Synth.startEngine(0);
	Synth.getSharedContext().enableTracking(true);
    	LineOut out = new LineOut();
    	final StagedVowelizer fof = new StagedVowelizer();
    	ImpulseOscillatorBL osc = new ImpulseOscillatorBL();
    	osc.output.connect(fof.input);
    	osc.frequency.set(100);
        osc.amplitude.set(1.0);
    	//LineIn in = new LineIn();
    	osc.output.connect(0, fof.input, 0);
    	fof.output.connect(0, out.input, 0);
    	fof.output.connect(0, out.input, 1);
    	fof.amplitude.set(10.0);
    	osc.start();
    	//in.start();
    	fof.start();
    	out.start();
    	java.awt.Frame f = new java.awt.Frame();
	f.add("North", new com.softsynth.jsyn.view11x.SoundTester(fof));
        java.awt.Panel p = new java.awt.Panel();
        p.setLayout(new java.awt.GridLayout(0, 1));
        p.add(new com.softsynth.jsyn.view11x.PortFader(fof.rate, 0.2, 0.1, 2.0));
        p.add(new com.softsynth.jsyn.view11x.PortFader(osc.frequency, 60, 20, 200.0));
        p.add(new com.softsynth.jsyn.view11x.PortFader(osc.amplitude, 1.0, 0.0, 1.0));
        f.add("South", p);
	f.pack();
    	f.setVisible(true);
    	f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					Synth.stopEngine();
					System.exit(0);
				}
			}
		);
    }
}